﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;

namespace Product_API_Availability_Request
{
    class Program
    {
        static void Main(string[] args)
        {
            RentacarAvail();
        }

        static void RentacarAvail()
        {
            // 1. Parameters (parameters you may specify on the request)

            string User = "InsertUserHere";
            string Pass = "InsertPassHere";

            string Language = "en";

            int AgeDriver = 24;

            DateTime StartDate = DateTime.Now.AddDays(30);
            DateTime EndDate = DateTime.Now.AddDays(31).AddHours(2);

            string CountryOfResidence = "ES";

            int OriginZone = 15011;
            int DestinationZone = 15011; // Destination code for Palma de Mallorca in the test environment

            // 2. Initialization (this code initializes all of the neccesaries nodes and propestier of the RentacarAvail request)

            WebServiceJP.JP_RentacarAvail AvailRQ = new WebServiceJP.JP_RentacarAvail();
            AvailRQ.Version = "1.1";
            AvailRQ.Language = Language;

            // 2.1. Login

            WebServiceJP.JP_Login Login = new WebServiceJP.JP_Login();
            Login.Email = User;
            Login.Password = Pass;
            AvailRQ.Login = Login;

            // 2.2. Request

            WebServiceJP.JP_RentacarRequest ProductRequest = new WebServiceJP.JP_RentacarRequest();
            WebServiceJP.JP_SearchSegmentsRentacar SearchSegmentsRentacar = new WebServiceJP.JP_SearchSegmentsRentacar();
            WebServiceJP.JP_SearchSegmentRentacar SearchSegmentRentacar = new WebServiceJP.JP_SearchSegmentRentacar();

            SearchSegmentRentacar.CountryOfResidence = CountryOfResidence;
            SearchSegmentRentacar.AgeDriver = AgeDriver;

            SearchSegmentRentacar.Start = StartDate;
            SearchSegmentRentacar.End = EndDate;

            SearchSegmentRentacar.HourSpecified = true;
            SearchSegmentRentacar.Hour = StartDate;
            SearchSegmentRentacar.HourEndSpecified = true;
            SearchSegmentRentacar.HourEnd = EndDate;

            SearchSegmentRentacar.OriginZoneSpecified = true;
            SearchSegmentRentacar.OriginZone = OriginZone;
            SearchSegmentRentacar.DestinationZoneSpecified = true;
            SearchSegmentRentacar.DestinationZone = DestinationZone;

            SearchSegmentsRentacar.SearchSegmentRentacar = SearchSegmentRentacar;
            ProductRequest.SearchSegmentsRentacar = SearchSegmentsRentacar;

            AvailRQ.RentacarRequest = ProductRequest;

            // 2.3. AdvancedOptions (optional parameters)

            WebServiceJP.JP_RentacarAvailAdvancedOptions AdvancedOptions = new WebServiceJP.JP_RentacarAvailAdvancedOptions();
            AdvancedOptions.ShowBreakdownPriceSpecified = true;
            AdvancedOptions.ShowBreakdownPrice = true;

            AvailRQ.AdvancedOptions = AdvancedOptions;

            // 3. Avail Request (here we prepare and send the Avail request)

            WebServiceJP.AvailTransactionsClient serv = new WebServiceJP.AvailTransactionsClient();

            Console.WriteLine(string.Format("- AVAIL REQUEST"));
            Console.WriteLine(string.Empty);
            Console.WriteLine(string.Format("Endpoint: {0}", serv.Endpoint.Address));
            Console.WriteLine(string.Format("User: {0}", User));
            Console.WriteLine(string.Format("Pass: {0}", Pass));
            Console.WriteLine(string.Format("Dates: {0} - {1}", StartDate.ToShortDateString(), EndDate.ToShortDateString()));
            Console.WriteLine(string.Format("OriginCode: {0}", OriginZone));
            Console.WriteLine(string.Format("DestinationCode: {0}", DestinationZone));
            Console.WriteLine(string.Empty);

            WebServiceJP.JP_AvailResponseRS RST = serv.RentacarAvail(AvailRQ);

            // 4. Avail Response (here we process the Avail response)

            Console.WriteLine(string.Format("- AVAIL RESPONSE"));
            Console.WriteLine(string.Empty);
            if (RST != null && RST.Results != null)
            {
                int OptionsCount = 0;
                int Count = 0;
                foreach (WebServiceJP.JP_RentacarResult item in RST.Results.Items)
                {

                    Count += 1;
                    foreach (WebServiceJP.JP_RentacarOption rentacarOption in item.RentacarOptions)
                        OptionsCount += 1;
                }

                Console.WriteLine(string.Format("OK"));
                Console.WriteLine(string.Format("Product: {0}", Count));
                Console.WriteLine(string.Format("ProductOptions: {0}", OptionsCount));
            }
            else if (RST != null && RST.Errors != null)
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: {0}", RST.Errors[0].Code));
            }
            else
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: UNKNOWN ERROR"));
            }

            // 5. XML serialization examples (for when you require for the XML logs any of the requests/reponses)

            string rqXML = serializeXML<WebServiceJP.JP_RentacarAvail>(AvailRQ);
            string rsXML = serializeXML<WebServiceJP.JP_AvailResponseRS>(RST);

            Console.ReadLine();
        }

        static string serializeXML<T>(object xmlobject)
        {
            XmlSerializer xsSubmit = new XmlSerializer(typeof(T));
            var subReq = xmlobject;

            using (var sww = new StringWriter())
            {
                using (XmlWriter writer = XmlWriter.Create(sww))
                {
                    xsSubmit.Serialize(writer, subReq);
                    return sww.ToString();
                }
            }
        }
    }
}
